/*
    This file is part of lorcon

    lorcon is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    lorcon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with lorcon; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Copyright (c) 2005 dragorn and Joshua Wright
*/

#ifndef __LORCON_FORGE_H__
#define __LORCON_FORGE_H__

#include "config.h"

#ifdef HAVE_STDINT_H
#include <stdint.h>
#endif
#ifdef HAVE_INTTYPES_H
#include <inttypes.h>
#endif

#include "ieee_80211.h"

// Tagged parameter in a frame
typedef struct lcpf_tagparm {
    uint8_t tag_num;
    uint8_t tag_len;
    uint8_t tag_data[0];
} __attribute__ ((packed));

#define lcpf_genstate_init          0
#define lcpf_genstate_

typedef struct lcpf_packet {
    // Generation state (can't add tagparms after raw payload, for example)
    uint8_t gen_state;
    // Raw packet buffer
    uint8_t *raw_buf;
    // Current length and maximum possible length
    unsigned int len;
    unsigned int max_len;
    // Pointer to where to append data
    uint8_t *append_ptr;

    ieee802_hdr *ieee_header;
    uint8_t *data_payload;

    struct {
        uint8_t nr_tagparms;
        lcpf_tagparm *tagparms;
    } tagged_parameters;

    struct ieee80211_mgmt *mgmt_header;
};

// Forge a random mac within a reasonable OUI
void lcpf_createrandmac(uint8_t *addr);

// Initialize a raw frame with the provided info
lcpf_packet *lcpf_packet_init(int in_carriermax, uint8_t in_type, 
							  uint8_t in_subtype, uint8_t *in_addr0, 
							  uint8_t *in_addr1, uint8_t *in_addr2);

// Add a tagged parameter to a packet
int lcpf_packet_addtagparm(int in_parm, int in_len, uint8_t *in_data, 
						   lcpf_packet *mod_packet);

// Add a data payload
int lcpf_packet_addpayload(int in_len, uint8_t *in_data, lcpf_packet *mod_packet);

#endif

